@@ -37,15 +37,71 @@ import librosa
 import warnings
 import scipy.io.wavfile as wave
 from scipy import interpolate
+import scipy.signal
 import matplotlib.pyplot as plt
+import matplotlib.patches as patches
 from ketos.utils import ensure_dir, morlet_func
 from ketos.data_handling.data_handling import read_wave
 from ketos.audio.annotation import AnnotationHandler
 from ketos.audio.utils.axis import LinearAxis
-from ketos.audio.base_audio import BaseAudio, segment_data
+from ketos.audio.base_audio import BaseAudioTime, segment_data
 import ketos.audio.utils.misc as aum
 
-class Waveform(BaseAudio):
+
+def plot(waveforms, labels="", figsize=(5,4), title="", offset=0, duration=None):
+    """ Plot one or several waveforms superimposed on one another.
+
+        Note: The resulting figure can be shown (fig.show())
+        or saved (fig.savefig(file_name))
+
+        Args:
+            waveforms: Waveform or list(Waveform)
+                Waveforms to be plotted
+            labels: str or list(str)
+                Labels used to identify the waveforms. 
+                Must have the same length as waveforms.
+            figsize: tuple
+                Figure size
+            title: str
+                Figure title.
+            offset, duration: float
+                Start time and length of the plotted segment in seconds. 
+                If not specified, the full waveform will be plotted.
+        
+        Returns:
+            fig: matplotlib.figure.Figure
+                Figure object.
+    """
+    if isinstance(waveforms, Waveform): waveforms = [waveforms]
+    if isinstance(labels, str): labels = [labels]
+
+    assert len(waveforms) == len(labels), "waveforms and labels must have the same length"
+
+    fig, ax = plt.subplots(nrows=1, ncols=1, figsize=figsize)
+
+    colors = [f"C{i}" for i in range(6)]
+    lstyles = ['-','--',':','-.']
+
+    for i,wf in enumerate(waveforms):
+        start = min(offset, wf.duration())
+        end = wf.duration()
+        if duration != None: end = min(end, start + duration)
+        wfc = wf.crop(start=start, end=end, make_copy=True)
+        col = colors[i%len(colors)]
+        lsty = lstyles[i%len(lstyles)]
+        x = np.linspace(start=start, stop=end, num=wfc.data.shape[0])
+        y = wfc.get_data()
+        ax.plot(x, y, label=labels[i], color=col, linestyle=lsty)
+        ax.set_xlabel(wfc.time_ax.label)
+        ax.set_ylabel('Amplitude')
+        ax.set_title(title)
+
+    if len(waveforms) > 1: ax.legend()
+
+    return fig
+
+
+class Waveform(BaseAudioTime):
     """ Audio signal
 
         Args:
@@ -62,6 +118,12 @@ class Waveform(BaseAudio):
                 Spectrogram label. Optional
             annot: AnnotationHandler
                 AnnotationHandler object. Optional
+            transforms: list(dict)
+                List of dictionaries, where each dictionary specifies the name of 
+                a transformation to be applied to this instance. For example,
+                {"name":"normalize", "mean":0.5, "std":1.0}
+            transform_log: list(dict)
+                List of transforms that have been applied to this instance
 
         Attributes:
             rate: float
@@ -79,8 +141,12 @@ class Waveform(BaseAudio):
                 Spectrogram label.
             annot: AnnotationHandler
                 AnnotationHandler object.
+            transform_log: list(dict)
+                List of transforms that have been applied to this instance
     """
-    def __init__(self, data, time_res=None, filename='', offset=0, label=None, annot=None, **kwargs):
+    def __init__(self, data, time_res=None, filename='', offset=0, label=None, annot=None, transforms=None,
+                    transform_log=None, **kwargs):
+
         assert time_res is not None or 'rate' in kwargs, "either time_res or rate must be specified"
 
         if time_res is None:
@@ -88,11 +154,23 @@ class Waveform(BaseAudio):
         else:
             self.rate = 1. / time_res
 
-        super().__init__(data=data, time_res=1./self.rate, ndim=1, filename=filename, offset=offset, label=label, annot=annot)
+        super().__init__(data=data, time_res=1./self.rate, filename=filename, offset=offset, label=label, annot=annot, 
+                            transform_log=transform_log, **kwargs)
+
+        self.allowed_transforms.update({'add_gaussian_noise': self.add_gaussian_noise, 
+                                        'bandpass_filter': self.bandpass_filter})
+        
+        self.apply_transforms(transforms)
+
+    def get_repres_attrs(self):
+        """ Get audio representation attributes """ 
+        attrs = super().get_repres_attrs()
+        attrs.update({'rate':self.rate, 'type':self.__class__.__name__})
+        return attrs
 
     @classmethod
     def from_wav(cls, path, channel=0, rate=None, offset=0, duration=None, resample_method='scipy',
-        id=None, normalize_wav=False, **kwargs):
+        id=None, normalize_wav=False, transforms=None, **kwargs):
         """ Load audio data from wave file.
 
             If `duration` (and `offset`) are specified and `offset + duration` exceeds the 
@@ -130,6 +208,10 @@ class Waveform(BaseAudio):
                 normalize_wav: bool
                     Normalize the waveform to have a mean of zero (mean=0) and a standard 
                     deviation of unity (std=1). Default is False.
+                transforms: list(dict)
+                    List of dictionaries, where each dictionary specifies the name of 
+                    a transformation to be applied to this instance. For example,
+                    {"name":"normalize", "mean":0.5, "std":1.0}
 
             Returns:
                 Instance of Waveform
@@ -144,8 +226,10 @@ class Waveform(BaseAudio):
                 >>> fig.savefig("ketos/tests/assets/tmp/audio_grunt1.png")
                 >>> plt.close(fig)
 
-                .. image:: ../../../../ketos/tests/assets/tmp/audio_grunt1.png
+                .. image:: ../../../ketos/tests/assets/tmp/audio_grunt1.png
         """
+        if transforms is None: transforms = []
+
         assert duration is None or duration >= 0, 'duration must be non-negative'
 
         # if 'id' is not specified, use the filename
@@ -184,6 +268,8 @@ class Waveform(BaseAudio):
                 duration += offset
                 duration = max(0, duration)
 
+        num_pad_left = max(0, num_pad_left)
+
         if duration is not None and duration == 0:
             data = np.zeros(num_pad_left, dtype=np.float64)
             if rate is None: rate = rate_orig
@@ -208,16 +294,15 @@ class Waveform(BaseAudio):
 
         # pad with zeros on the right, to achieve desired duration, if necessary
         if duration is not None:
-            num_pad_right = int(duration * rate - data.shape[0])
+            num_pad_right = max(0, int(duration * rate - data.shape[0]))
             if num_pad_right > 0 or num_pad_left > 0:
                 data = np.pad(data, pad_width=((num_pad_left,num_pad_right)), mode='constant')
                 warnings.warn("Waveform padded with zeros to achieve desired length", RuntimeWarning)
 
         if normalize_wav: 
-            std = np.std(data)
-            if std > 0: data = (data - np.mean(data)) / std
+            transforms.append({'name':'normalize','mean':0.0,'std':1.0})
 
-        return cls(rate=rate, data=data, filename=id, offset=offset)
+        return cls(rate=rate, data=data, filename=id, offset=offset, transforms=transforms, **kwargs)
 
     @classmethod
     def gaussian_noise(cls, rate, sigma, samples, filename="gaussian_noise"):
@@ -246,7 +331,7 @@ class Waveform(BaseAudio):
                 >>> fig.savefig("ketos/tests/assets/tmp/audio_noise.png")
                 >>> plt.close(fig)
 
-                .. image:: ../../../../ketos/tests/assets/tmp/audio_noise.png
+                .. image:: ../../../ketos/tests/assets/tmp/audio_noise.png
         """        
         assert sigma > 0, "sigma must be strictly positive"
 
@@ -293,7 +378,7 @@ class Waveform(BaseAudio):
                 >>> fig = wavelet1.plot()
                 >>> fig.savefig("ketos/tests/assets/tmp/morlet_standard.png")
 
-                .. image:: ../../../../ketos/tests/assets/tmp/morlet_standard.png
+                .. image:: ../../../ketos/tests/assets/tmp/morlet_standard.png
 
                 >>> # create another wavelet, but with frequency increasing linearly with time
                 >>> wavelet2 = Waveform.morlet(rate=100., frequency=3., width=2.0, dfdt=0.3)
@@ -302,7 +387,7 @@ class Waveform(BaseAudio):
                 >>> fig.savefig("ketos/tests/assets/tmp/morlet_dfdt.png")
                 >>> plt.close(fig)
 
-                .. image:: ../../../../ketos/tests/assets/tmp/morlet_dfdt.png
+                .. image:: ../../../ketos/tests/assets/tmp/morlet_dfdt.png
         """        
         if samples is None:
             samples = int(6 * width * rate)
@@ -350,7 +435,7 @@ class Waveform(BaseAudio):
                 >>> fig.savefig("ketos/tests/assets/tmp/cosine_audio.png")
                 >>> plt.close(fig)
 
-                .. image:: ../../../../ketos/tests/assets/tmp/cosine_audio.png
+                .. image:: ../../../ketos/tests/assets/tmp/cosine_audio.png
         """        
         N = int(duration * rate)
 
@@ -364,75 +449,6 @@ class Waveform(BaseAudio):
         
         return cls(rate=rate, data=np.array(y), filename=filename)
 
-    def get_attrs(self):
-        return {'rate':self.rate, 'type':self.__class__.__name__}
-
-    def get_data(self, id=0):
-        """ Get the underlying data numpy array.
-
-            Args:
-                id: int
-                    Audio signal ID. Only relevant if the Waveform object 
-                    contains multiple, stacked audio signals.
-
-            Returns:
-                d: numpy array
-                    Data
-        """
-        if id is None or np.ndim(self.data) == 1:
-            d = self.data
-        else:
-            d = self.data[:,id]
-
-        return d
-
-    def segment(self, window, step=None):
-        """ Divide the time axis into segments of uniform length, which may or may 
-            not be overlapping.
-
-            Window length and step size are converted to the nearest integer number 
-            of time steps.
-
-            If necessary, the audio signal will be padded with zeros at the end to 
-            ensure that all segments have an equal number of samples. 
-
-            Args:
-                window: float
-                    Length of each segment in seconds.
-                step: float
-                    Step size in seconds.
-
-            Returns:
-                segs: Waveform
-                    Stacked audio signals
-
-            Example:
-                >>> from ketos.audio.waveform import Waveform
-                >>> # create a morlet wavelet
-                >>> mor = Waveform.morlet(rate=100, frequency=5, width=0.5)
-                >>> mor.duration()
-                3.0
-                >>> # segment into 2-s wide frames, using a step size of 1 s
-                >>> segs = mor.segment(window=2., step=1.)
-                >>> # show the segments
-                >>> fig0 = segs.plot(0)
-                >>> fig0.savefig("ketos/tests/assets/tmp/morlet_segmented_0.png")
-                >>> fig1 = segs.plot(1)
-                >>> fig1.savefig("ketos/tests/assets/tmp/morlet_segmented_1.png")
-                >>> plt.close(fig0)
-                >>> plt.close(fig1)
-
-                .. image:: ../../../../ketos/tests/assets/tmp/morlet_segmented_0.png
-
-                .. image:: ../../../../ketos/tests/assets/tmp/morlet_segmented_1.png
-        """              
-        segs, filename, offset, label, annot = segment_data(self, window, step)
-
-        d = self.__class__(data=segs, rate=self.rate, filename=filename,\
-            offset=offset, label=label, annot=annot)
-
-        return d
-
     def to_wav(self, path, auto_loudness=True):
         """ Save audio signal to wave file
 
@@ -454,7 +470,7 @@ class Waveform(BaseAudio):
 
         wave.write(filename=path, rate=int(self.rate), data=(s*self.data).astype(dtype=np.int16))
 
-    def plot(self, id=0, show_annot=False):
+    def plot(self, show_annot=False, figsize=(5,4), label_in_title=True, append_title='', show_envelope=False):
         """ Plot the data with proper axes ranges and labels.
 
             Optionally, also display annotations as boxes superimposed on the data.
@@ -463,11 +479,16 @@ class Waveform(BaseAudio):
             or saved (fig.savefig(file_name))
 
             Args:
-                id: int
-                    ID of data array to be plotted. Only relevant if the object 
-                    contains multiple, stacked data arrays.
                 show_annot: bool
                     Display annotations
+                figsize: tuple
+                    Figure size
+                label_in_title: bool
+                    Include label (if available) in figure title
+                append_title: str
+                    Append this string to the title
+                show_envelope: bool
+                    Display envelope on top of signal
             
             Returns:
                 fig: matplotlib.figure.Figure
@@ -481,40 +502,45 @@ class Waveform(BaseAudio):
                 >>> fig = a.plot()
                 >>> plt.close(fig)
 
-                .. image:: ../../_static/morlet.png
+                .. image:: ../_static/morlet.png
         """
-        fig, ax = super().plot(id=id)
+        fig, ax = super().plot(figsize, label_in_title, append_title)
 
-        y = self.get_data(id)
+        y = self.get_data()
 
         x = np.linspace(start=0, stop=self.duration(), num=self.data.shape[0])
         ax.plot(x, y)
         ax.set_ylabel('Amplitude')
 
+        # superimpose envelope
+        if show_envelope:
+            z = np.abs(scipy.signal.hilbert(y))
+            ax.plot(x, z, color='C1')
+
         # superimpose annotation boxes
-        if show_annot: self._draw_annot_boxes(ax,id)
+        if show_annot: self._draw_annot_boxes(ax)
 
         #fig.tight_layout()
         return fig
 
-    def _draw_annot_boxes(self, ax, id=0):
+    def _draw_annot_boxes(self, ax):
         """Draws annotations boxes on top of the spectrogram
 
             Args:
                 ax: matplotlib.axes.Axes
                     Axes object
-                id: int
-                    Object ID
         """
-        annots = self.get_annotations(id=id)
+        annots = self.get_annotations()
         if annots is None: return
         y1, y2 = ax.get_ylim()
+        y1 *= 0.95
+        y2 *= 0.95
         for idx,annot in annots.iterrows():
             x1 = annot['start']
             x2 = annot['end']
-            box = patches.Rectangle((x1,y1),x2-x1,y2-y1,linewidth=1,edgecolor='C1',facecolor='none')
+            box = patches.Rectangle((x1,y1),x2-x1,y2-y1,linewidth=1,edgecolor='C3',facecolor='none')
             ax.add_patch(box)
-            ax.text(x1, y2, int(annot['label']), ha='left', va='bottom', color='C1')
+            ax.text(x1, y2, int(annot['label']), ha='left', va='bottom', color='C3')
 
     def append(self, signal, n_smooth=0):
         """ Append another audio signal to the present instance.
@@ -549,7 +575,7 @@ class Waveform(BaseAudio):
                 >>> fig.savefig("ketos/tests/assets/tmp/morlet_cosine.png")
                 >>> plt.close(fig)
 
-                .. image:: ../../../../ketos/tests/assets/tmp/morlet_cosine.png
+                .. image:: ../../../ketos/tests/assets/tmp/morlet_cosine.png
         """   
         assert self.rate == signal.rate, "Cannot merge audio signals with different sampling rates."
 
@@ -603,12 +629,70 @@ class Waveform(BaseAudio):
                 >>> fig.savefig("ketos/tests/assets/tmp/morlet_w_noise.png")
                 >>> plt.close(fig)
 
-                .. image:: ../../../../ketos/tests/assets/tmp/morlet_wo_noise.png
+                .. image:: ../../../ketos/tests/assets/tmp/morlet_wo_noise.png
 
-                .. image:: ../../../../ketos/tests/assets/tmp/morlet_w_noise.png
+                .. image:: ../../../ketos/tests/assets/tmp/morlet_w_noise.png
         """
         noise = Waveform.gaussian_noise(rate=self.rate, sigma=sigma, samples=len(self.data))
         self.add(noise)
+        self.transform_log.append({'name':'add_gaussian_noise', 'sigma':sigma})
+
+    def bandpass_filter(self, freq_min=None, freq_max=None, N=3):
+        """ Apply a lowpass, highpass, or bandpass filter to the signal.
+
+            Uses SciPy's implementation of an Nth-order digital Butterworth filter.
+
+            The critical frequencies, freq_min and freq_max, correspond to the points 
+            at which the gain drops to 1/sqrt(2) that of the passband (the “-3 dB point”).
+
+            Args:
+                freq_min: float
+                    Lower limit of the frequency window in Hz.
+                    (Also sometimes referred to as the highpass frequency).
+                    If None, a lowpass filter is applied. 
+                freq_max: float
+                    Upper limit of the frequency window in Hz.
+                    (Also sometimes referred to as the lowpass frequency)
+                    If None, a highpass filter is applied. 
+                N: int
+                    The order of the filter. The default value is 3.
+
+            Example:
+                >>> from ketos.audio.waveform import Waveform
+                >>> # create a Cosine waves with frequencies of 7 and 14 Hz
+                >>> cos = Waveform.cosine(rate=1000., frequency=7.)
+                >>> cos14 = Waveform.cosine(rate=1000., frequency=14.)
+                >>> cos.add(cos14)
+                >>> # show combined signal
+                >>> fig = cos.plot()
+                >>> fig.savefig("ketos/tests/assets/tmp/cosine_double_audio.png")
+                >>> plt.close(fig)
+                >>> # apply 10 Hz highpass filter
+                >>> cos.bandpass_filter(freq_max=10)
+                >>> # show filtered signal
+                >>> fig = cos.plot()
+                >>> fig.savefig("ketos/tests/assets/tmp/cosine_double_hp_audio.png")
+                >>> plt.close(fig)
+
+                .. image:: ../../../ketos/tests/assets/tmp/cosine_double_audio.png
+
+                .. image:: ../../../ketos/tests/assets/tmp/cosine_double_hp_audio.png
+        """
+        if freq_min is None and freq_max is None: return
+
+        if freq_min is None: 
+            Wn = freq_max
+            btype = 'lowpass'
+        elif freq_max is None: 
+            Wn = freq_min            
+            btype = 'highpass'
+        else: 
+            Wn = (freq_min, freq_max)            
+            btype = 'bandpass'
+
+        b,a = scipy.signal.butter(N=N, Wn=Wn, btype=btype, fs=self.rate)
+        self.data = scipy.signal.filtfilt(b, a, self.data)
+        self.transform_log.append({'name':'bandpass_filter', 'freq_min':freq_min, 'freq_max':freq_max, 'N':N})
 
     def add(self, signal, offset=0, scale=1):
         """ Add the amplitudes of the two audio signals.
@@ -642,7 +726,7 @@ class Waveform(BaseAudio):
                 >>> fig.savefig("ketos/tests/assets/tmp/morlet_cosine_added.png")
                 >>> plt.close(fig)
 
-                .. image:: ../../../../ketos/tests/assets/tmp/morlet_cosine_added.png
+                .. image:: ../../../ketos/tests/assets/tmp/morlet_cosine_added.png
         """
         assert self.rate == signal.rate, "Cannot add audio signals with different sampling rates."
 
@@ -663,37 +747,30 @@ class Waveform(BaseAudio):
         bins = signal.data.shape[0]
         self.data[b:b+bins] = self.data[b:b+bins] + scale * signal.data
 
-    def resample(self, new_rate):
+    def resample(self, new_rate, resample_method='scipy'):
         """ Resample the acoustic signal with an arbitrary sampling rate.
 
-        Note: Code adapted from Kahl et al. (2017)
-              Paper: http://ceur-ws.org/Vol-1866/paper_143.pdf
-              Code:  https://github.com/kahst/BirdCLEF2017/blob/master/birdCLEF_spec.py  
-
         Args:
             new_rate: int
                 New sampling rate in Hz
+            resample_method: str
+                Resampling method. Only relevant if `rate` is specified. Options are
+                    * kaiser_best
+                    * kaiser_fast
+                    * scipy (default)
+                    * polyphase
+                    
+                See https://librosa.github.io/librosa/generated/librosa.core.resample.html 
+                for details on the individual methods.
         """
         if len(self.data) < 2:
             self.rate = new_rate
 
         else:                
-            orig_rate = self.rate
-            sig = self.data
-
-            duration = sig.shape[0] / orig_rate
-
-            time_old  = np.linspace(0, duration, sig.shape[0])
-            time_new  = np.linspace(0, duration, int(sig.shape[0] * new_rate / orig_rate))
-
-            interpolator = interpolate.interp1d(time_old, sig.T)
-            new_audio = interpolator(time_new).T
-
-            new_sig = np.round(new_audio).astype(sig.dtype)
-
+            self.data = librosa.core.resample(self.get_data(), orig_sr=self.rate, target_sr=new_rate, res_type=resample_method)
             self.rate = new_rate
-            self.data = new_sig
-            self.time_ax = LinearAxis(bins=self.data.shape[0], extent=(0., self.data.shape[0] / self.rate), label='Time (s)') #new time axis
+
+        self.time_ax = LinearAxis(bins=self.data.shape[0], extent=(0., self.data.shape[0] / self.rate), label='Time (s)') #new time axis
 
 
 def _smoothclamp(x, mi, mx): 
