@@ -78,10 +78,10 @@ def map_detection_to_time(det_start, det_end, batch_start_timestamp, batch_end_t
             det_end: int
                 The detection end (the last score that is part of the detection) expressed as an index in the scores vector. 
             batch_start_timestap:float
-                 The timestamp (in seconds from the beginning of the file) of the first score in the scores vector
+                The timestamp (in seconds from the beginning of the file) of the first score in the scores vector
                 (i.e.: the score of the first input spectrogram in that batch)
             batch_end_timestap:float
-                 The timestamp (in seconds from the beginning of the file) of the last score in the scores vector
+                The timestamp (in seconds from the beginning of the file) of the last score in the scores vector
                 (i.e.: the score of the last input spectrogram in that batch)
             step: float
                 The time interval(in seconds) between the starts of each contiguous input spectrogram.
@@ -98,8 +98,6 @@ def map_detection_to_time(det_start, det_end, batch_start_timestamp, batch_end_t
         Returns:
             time_start, duration:float
                 The corresponding start (in seconds from the beggining of the file) and duration
-
-
     """
     if det_end < det_start:
         raise ValueError("'det_end' cannot be lower than 'det_start'")
@@ -120,7 +118,8 @@ def map_detection_to_time(det_start, det_end, batch_start_timestamp, batch_end_t
 def group_detections(scores_vector, batch_support_data, buffer=0.0, step=0.5, spec_dur=3.0, threshold=0.5):
     """ Groups time steps with a detection score above the specified threshold.
 
-        Consecutive detections are grouped into one single detection represented by the time interval (start-end, in seconds from beginning of the file).
+        Consecutive detections are grouped into one single detection represented by the 
+        time interval (start-end, in seconds from beginning of the file).
 
         Args:
             scores_vector: numpy array
@@ -134,40 +133,61 @@ def group_detections(scores_vector, batch_support_data, buffer=0.0, step=0.5, sp
                 For example, a step=0.5 indicates that the first spectrogram starts at time 0.0s (from the beginning of the audio file), the second at 0.5s, etc.
             spec_dur: float
                 The duration of each spectrogram in seconds
-            threshold: float
+            threshold: float or list of floats
                 Minimum score value for a time step to be considered as a detection.
                 
         Returns:
-            det_timestamps:list of tuples
+            det_timestamps: list of tuples
                 The detections time stamp. Each item in the list is a tuple with the filename, start time, duration and score for that detection.
-                The filename corresponds to the file where the detection started.          
+                The filename corresponds to the file where the detection started. 
+                If a list of threshold values is specified, the returned object will be a list of lists with len(det_timestamps) = len(thresholds)         
     """
+    thresholds = threshold if isinstance(threshold, list) else [threshold]
 
-    det_vector = np.where(scores_vector >= threshold, 1.0, 0.0)
-    det_timestamps = []
-    within_det = False
-    filename_vector = batch_support_data[:,0]
+    det_timestamps = [[] for _ in range(len(thresholds))]
 
-    for det_index,det_value in enumerate(det_vector):
+    for i in range(len(thresholds)):
+        thres = thresholds[i]
 
-        if det_value == 1.0 and not within_det: #start new detection
-            start = det_index
-            within_det = True
+        det_vector = np.where(scores_vector >= thres, 1.0, 0.0)
+        within_det = False
+        filename_vector = batch_support_data[:,0]
 
-        if (det_value == 0 or filename_vector[det_index] != filename_vector[max(0,det_index-1)]) and within_det: #end current detection
-            end = det_index - 1
-            within_det = False
-            filename = filename_vector[start]
-            # From all timestamps within the batch, select only the timestamps for the file containing the detection start
-            file_timestamps = batch_support_data[filename_vector==filename]
-            batch_start_timestamp = float(file_timestamps[0,1])
-            batch_end_timestamp = float(file_timestamps[-1,1])
-            time_start, duration = map_detection_to_time(start, end, step=step, spec_dur=spec_dur, batch_start_timestamp=batch_start_timestamp, batch_end_timestamp=batch_end_timestamp, buffer=buffer)
-            score = np.average(scores_vector[start:(end + 1)])
-            if np.isnan(score) and end==start:
-                score = scores_vector[start]
+        prev_det_index = 0
+        for det_index,det_value in enumerate(det_vector):
+
+            is_new_file = (filename_vector[det_index] != filename_vector[max(0,prev_det_index)])
+            is_end_of_vector = (det_index == len(det_vector) - 1)
+
+            if det_value == 1.0 and not within_det: #start new detection
+                start = det_index
+                within_det = True
             
-            det_timestamps.append((filename, time_start, duration, score))
+            if (det_value == 0 or is_new_file or is_end_of_vector) and within_det: #end current detection
+                if is_end_of_vector:
+                    end = det_index
+                else:
+                    end = prev_det_index
+                
+                within_det = False
+                filename = filename_vector[start]
+
+                # From all timestamps within the batch, select only the timestamps for the file containing the detection start
+                file_timestamps = batch_support_data[filename_vector==filename]
+
+                batch_start_timestamp = float(file_timestamps[0,1])
+                batch_end_timestamp = float(file_timestamps[-1,1])
+                time_start, duration = map_detection_to_time(start, end, step=step, spec_dur=spec_dur, batch_start_timestamp=batch_start_timestamp, batch_end_timestamp=batch_end_timestamp, buffer=buffer)
+                score = np.average(scores_vector[start:(end + 1)])
+                if np.isnan(score) and end==start:
+                    score = scores_vector[start]
+                
+                det_timestamps[i].append((filename, time_start, duration, score))
+
+            prev_det_index = det_index #update index of previous instance
+
+    if not isinstance(threshold, list): 
+        det_timestamps = det_timestamps[0]
 
     return det_timestamps
 
@@ -188,7 +208,7 @@ def process_batch(batch_data, batch_support_data, model, buffer=0, step=0.5, spe
                 For example, a step=0.5 indicates that the first spectrogram starts at time 0.0s (from the beginning of the audio file), the second at 0.5s, etc.
             spec_dur: float
                 The duration of each input spectrogram in seconds
-            threshold: float
+            threshold: float or list of floats
                 Minimum score value for a time step to be considered as a detection.
             win_len:int
                 The windown length for the moving average. Must be an odd integer. The default value is 5.
@@ -201,8 +221,11 @@ def process_batch(batch_data, batch_support_data, model, buffer=0, step=0.5, spe
             batch_detections: list
                 An array with all the detections in the batch. Each detection (first dimension) consists of the filename, start, duration and score.
                 The start is given in seconds from the beginning of the file and the duration in seconds. 
+                If a list of threshold values is specified, the returned object will be a list of arrays with len(batch_detections) = len(thresholds)         
 
     """
+    thresholds = threshold if isinstance(threshold, list) else [threshold]
+
     scores = model.run_on_batch(batch_data, return_raw_output=True)
 
     if win_len == 1:
@@ -211,15 +234,24 @@ def process_batch(batch_data, batch_support_data, model, buffer=0, step=0.5, spe
         scores = compute_avg_score(scores[:,1], win_len=win_len)
         
     if group == True:
-        batch_detections = group_detections(scores, batch_support_data, buffer=buffer, step=step, spec_dur=spec_dur, threshold=threshold) 
+        batch_detections = group_detections(scores, batch_support_data, buffer=buffer, step=step, spec_dur=spec_dur, threshold=thresholds) 
 
     else:
-        threshold_indices = scores >= threshold
-        batch_detections = np.vstack([batch_support_data[threshold_indices,0], batch_support_data[threshold_indices,1], np.repeat(spec_dur, sum(threshold_indices)), scores[threshold_indices]])
-        if batch_detections.shape[1] == 0:
-            batch_detections = []
-        else:
-            batch_detections = [(d[0], float(d[1]), float(d[2]), float(d[3])) for d in batch_detections.T]      
+        batch_detections = []
+
+        for thres in thresholds:
+            threshold_indices = scores >= thres
+            batch_det = np.vstack([batch_support_data[threshold_indices,0], batch_support_data[threshold_indices,1], np.repeat(spec_dur, sum(threshold_indices)), scores[threshold_indices]])
+            if batch_det.shape[1] == 0:
+                batch_det = []
+            else:
+                batch_det = [(d[0], float(d[1]), float(d[2]), float(d[3])) for d in batch_det.T]
+
+            batch_detections.append(batch_det)
+
+
+    if not isinstance(threshold, list): 
+        batch_detections = batch_detections[0]
 
     return batch_detections
 
@@ -242,16 +274,10 @@ def process(provider, **kwargs):
         return process_batch_generator(provider, **kwargs)
 
 
-def process_audio_loader(audio_loader, model, batch_size=128, threshold=0.5, buffer=0, win_len=1, group=False, progress_bar=False):
+def process_audio_loader(audio_loader, model, batch_size=128, threshold=0.5, buffer=0, win_len=1, group=False, progress_bar=False, merge=False):
     """ Use an audio_loader object to compute spectrogram from the audio files and process them with the trained classifier.
 
-        The resulting .csv is separated by commas, with each row representing one detection and has the following columns:
-            filename: The name of the audio file where the detection was registered. 
-                      In case the detection starts in one file and ends in the next,
-                      the name of the first file is registered.
-            start:    Start of the detection (in seconds from the beginning of the file)
-            end:      End of the detection (in seconds from the beginning of the file)
-            score:    The sore given to the detection by the trained neural network ([0-1])
+        TODO: Improve the implementation so that it is not limited to audio_loader.batch_size = 1
 
         Args:
             audio_loader: a ketos.audio.audio_loader.AudioFrameLoader object
@@ -260,7 +286,7 @@ def process_audio_loader(audio_loader, model, batch_size=128, threshold=0.5, buf
                 The ketos trained classifier
             batch_size:int
                 The number of spectrogram to process at a time.
-            threshold: float
+            threshold: float or list of floats
                 Minimum score value for a time step to be considered as a detection.
             buffer: float
                 Time (in seconds) to be added around the detection
@@ -272,12 +298,20 @@ def process_audio_loader(audio_loader, model, batch_size=128, threshold=0.5, buf
                 average score of all spectrograms comprising the detection event.
             progress_bar: bool
                 Show progress bar.  
+            merge: bool
+                Apply :func:`merge_overlapping_detections` to the detections before they are returned. Default is False.
 
         Returns:
             detections: list
-                List of detections
+                List of detections.
+                If a list of threshold values is specified, the returned object will be a list of lists with len(detections) = len(thresholds)         
     """
     assert isinstance(win_len, int) and win_len%2 == 1, 'win_len must be an odd integer'
+
+    assert audio_loader.batch_size == 1, "audio_loader must have batch size of 1"
+
+    thresholds = threshold if isinstance(threshold, list) else [threshold]
+        
     n_extend = int((win_len - 1) / 2)
 
     n_batches = audio_loader.num() // batch_size
@@ -290,7 +324,7 @@ def process_audio_loader(audio_loader, model, batch_size=128, threshold=0.5, buf
     else:
         batch_sizes = [batch_size + n_extend] + [batch_size + 2 * n_extend for _ in range(n_batches - 2)] + [last_batch_size + n_extend]
 
-    detections = []
+    detections = [[] for _ in range(len(thresholds))]
     specs_prev_batch = []
     duration = None
     step = 0
@@ -323,24 +357,26 @@ def process_audio_loader(audio_loader, model, batch_size=128, threshold=0.5, buf
         batch_support_data = np.array(batch_support_data)
         batch_data = np.array(batch_data)
 
-        batch_detections = process_batch(batch_data=batch_data, batch_support_data=batch_support_data, model=model, threshold=threshold, buffer=buffer, step=step, spec_dur=duration, win_len=win_len, group=group)
-        if len(batch_detections) > 0: detections += batch_detections
+        batch_detections = process_batch(batch_data=batch_data, batch_support_data=batch_support_data, model=model, threshold=thresholds, 
+                                        buffer=buffer, step=step, spec_dur=duration, win_len=win_len, group=group)
+
+        for i in range(len(thresholds)):
+            if len(batch_detections[i]) > 0: detections[i] += batch_detections[i]
+
+
+    if merge:
+        for i in range(len(thresholds)):
+            detections[i] = merge_overlapping_detections(detections[i])
+
+    if not isinstance(threshold, list): 
+        detections = detections[0]
 
     return detections
 
 
-def process_batch_generator(batch_generator, model, duration=3.0, step=0.5, threshold=0.5, buffer=0, win_len=1, group=False):
+def process_batch_generator(batch_generator, model, duration=3.0, step=0.5, threshold=0.5, buffer=0, win_len=1, group=False, merge=False):
     """ Use a batch_generator object to process pre-computed spectrograms stored in an HDF5 database with the trained classifier.
 
-        The resulting .csv is separated by commas, with each row representing one detection and has the following columns:
-            filename: The name of the audio file where the detection was registered. 
-                      In case the detection starts in one file and ends in the next,
-                      the name of the first file is registered.
-            start:    Start of the detection (in seconds from the beginning of the file)
-            end:      End of the detection (in seconds from the beginning of the file)
-            score:    The sore given to the detection by the trained neural network ([0-1])
-
-
         Args:
             batch_generator: a ketos.data_handling.data_feeding.BatchGenerator object
                 A batch_generator that loads pre-computed spectrograms from the a HDF5 database files as requested
@@ -351,7 +387,7 @@ def process_batch_generator(batch_generator, model, duration=3.0, step=0.5, thre
             step: float
                 The time interval(in seconds) between the starts of each contiguous input spectrogram.
                 For example, a step=0.5 indicates that the first spectrogram starts at time 0.0s (from the beginning of the audio file), the second at 0.5s, etc.
-            threshold: float
+            threshold: float or list of floats
                 Minimum score value for a time step to be considered as a detection.
             buffer: float
                 Time (in seconds) to be added around the detection
@@ -361,18 +397,34 @@ def process_batch_generator(batch_generator, model, duration=3.0, step=0.5, thre
                 If False, return the filename, start, duration and scores for each spectrogram with score above the threshold. In this case, the duration will always be the duration of a single spectrogram.
                 If True (default), average scores over(overlapping) spectrograms and group detections that are immediatelly next to each other. In this case, the score given for that detection will be the
                 average score of all spectrograms comprising the detection event.
+            merge: bool
+                Apply :func:`merge_overlapping_detections` to the detections before they are returned. Default is False.
 
         Returns:
             detections: list
-                List of detections
+                List of detections.
+                If a list of threshold values is specified, the returned object will be a list of lists with len(detections) = len(thresholds)         
     """ 
-    detections = []   
+    thresholds = threshold if isinstance(threshold, list) else [threshold]
+
+    detections = [[] for _ in range(len(thresholds))]
+
     for b in range(batch_generator.n_batches):
         batch_data, batch_support_data = next(batch_generator)
 
-        batch_detections = process_batch(batch_data=batch_data, batch_support_data=batch_support_data, model=model, threshold=threshold, buffer=buffer, spec_dur=duration, step=step, win_len=win_len, group=group)
-        if len(batch_detections) > 0: detections += batch_detections
+        batch_detections = process_batch(batch_data=batch_data, batch_support_data=batch_support_data, model=model, threshold=thresholds, 
+                                        buffer=buffer, spec_dur=duration, step=step, win_len=win_len, group=group)
         
+        for i in range(len(thresholds)):
+            if len(batch_detections[i]) > 0: detections[i] += batch_detections[i]
+
+
+    if merge:
+        for i in range(len(thresholds)):
+            detections[i] = merge_overlapping_detections(detections[i])
+
+    if not isinstance(threshold, list): 
+        detections = detections[0]
 
     return detections
 
@@ -404,10 +456,11 @@ def transform_batch(x,y):
 
 
 def merge_overlapping_detections(detections):
-    """ Merge overlapping detection groups
-        
-        Note: The annotations for each file are assumed to be sorted by start time in increasing order.
+    """ Merge overlapping or adjacent detections.
 
+        The score of the merged detection is computed as the average of the individual detection scores.
+
+        Note: The detections are assumed to be sorted by start time in chronological order.
 
         Args:
             detections: numpy.array
@@ -433,9 +486,12 @@ def merge_overlapping_detections(detections):
         end_prev = start_prev + duration_prev        
         score_prev = merged[-1][3]
         
-        if start > end_prev or filename != filename_prev:#do not overlap
+        # detections do not overlap, nor are they adjacent
+        if start > end_prev or filename != filename_prev:
             merged.append(detections[i])
-        else:#do overlap
+
+        # detections overlap, or adjacent to one another
+        else:
             merged_dur = max(end, end_prev) - min(start,start_prev)
             avg_score = 0.5 * (score + score_prev) 
             merged_det = (filename, start_prev, merged_dur, avg_score)
@@ -446,6 +502,17 @@ def merge_overlapping_detections(detections):
 def save_detections(detections, save_to):
     """ Save the detections to a csv file
 
+        The resulting .csv is separated by commas, with each row representing one detection and has the following columns:
+
+            filename: The name of the audio file where the detection was registered. In case the detection starts in one file and ends in the next, \
+                      the name of the first file is registered.
+
+            start:    Start of the detection (in seconds from the beginning of the file)
+
+            duration: Length of the detection (in seconds)
+
+            score:    The sore given to the detection by the trained neural network ([0-1])
+
         Args:
             detections: numpy.array
                 List of detections
