@@ -35,7 +35,7 @@ import ketos.audio.utils.misc as ap
 from ketos.data_handling.data_handling import to1hot
 from ketos.data_handling.data_feeding import BatchGenerator
 from ketos.audio.waveform import Waveform
-from ketos.audio.utils.axis import LinearAxis, Log2Axis
+from ketos.audio.utils.axis import LinearAxis, Log2Axis, MelAxis
 import ketos.audio.base_audio as aba
 
 path_to_assets = os.path.join(os.path.dirname(__file__),"assets")
@@ -76,6 +76,15 @@ def const_wave():
     signal = np.ones(len(x))
     return sampling_rate, signal
 
+@pytest.fixture
+def growing_sine_wave():
+    sampling_rate = 1000
+    frequency = 18
+    duration = 3
+    x = np.arange(duration * sampling_rate)
+    signal = x / np.max(x) * np.sin(2 * np.pi * frequency * x / sampling_rate) 
+    return sampling_rate, signal
+
 @pytest.fixture
 def sine_wave_file(sine_wave):
     """Create a .wav with the 'sine_wave()' fixture
@@ -167,6 +176,24 @@ def const_wave_file(const_wave):
     yield wav_file
     os.remove(wav_file)
 
+@pytest.fixture
+def growing_sine_wave_file(growing_sine_wave):
+    """Create a .wav with the 'growing_sine_wave()' fixture
+    
+       The file is saved as tests/assets/growing_sine_wave.wav.
+       When the tests using this fixture are done, 
+       the file is deleted.
+
+       Yields:
+            wav_file : str
+                A string containing the path to the .wav file.
+    """
+    wav_file = os.path.join(path_to_assets, "growing_sine_wave.wav")
+    rate, sig = growing_sine_wave
+    sf.write(wav_file, sig, rate)    
+    yield wav_file
+    os.remove(wav_file)
+
 @pytest.fixture
 def image_2x2():
     image = np.array([[1,2],[3,4]], np.float32)
@@ -411,6 +438,18 @@ def log2_axis_8_16():
     ax = Log2Axis(bins=8*16, bins_per_oct=16, min_value=10.)
     return ax
 
+@pytest.fixture
+def mel_axis_40_500():
+    """ Create a Mel frequency axis with with 40 bins (filters)
+        for a maximum frequncy of 500 Hz
+
+        Yields:
+            ax: MelAxis
+                Axis object
+    """
+    ax = MelAxis(num_filters=40, freq_max=500)
+    return ax
+
 @pytest.fixture
 def spec_image_with_attrs():
     """ Creates a spectrogram image with shape (20,10) and random pixel values, 
@@ -433,6 +472,20 @@ def spec_image_with_attrs():
 @pytest.fixture
 def base_audio_1d():
     """ Create a simple 1d BaseAudio object with value 1 everywhere, length 
+        of 10 s, filename 'x', offset of 2 s, and label 13.
+
+        Yields:
+            o: BaseAudio
+                BaseAudio object
+    """
+    N = 10000
+    d = np.ones(N)
+    o = aba.BaseAudio(data=d, filename='x', offset=2., label=13, duration=10.)
+    return o, d
+
+@pytest.fixture
+def base_audio_time_1d():
+    """ Create a simple 1d BaseAudioTime object with value 1 everywhere, length 
         of 10 s, time resolution of 0.001 s, filename 'x', offset of 2 s, and label 13.
 
         Yields:
@@ -441,28 +494,12 @@ def base_audio_1d():
     """
     N = 10000
     d = np.ones(N)
-    o = aba.BaseAudio(time_res=0.001, data=d, ndim=1, filename='x', offset=2., label=13)
-    return o, d
-
-@pytest.fixture
-def base_audio_1d_stacked():
-    """ Create a 1d BaseAudio object consisting of three stacked arrays (with values 1,2,3 
-        everywhere), and having a length of 10 s, time resolution of 0.001 s, filename 'x',
-        'y','z', offset of 2 s, and label 13.
-
-        Yields:
-            o: BaseAudio
-                BaseAudio object
-    """
-    N = 10000
-    d = np.ones((N,3))
-    d[:,1] = 2
-    d[:,2] = 3
-    o = aba.BaseAudio(time_res=0.001, data=d, ndim=1, filename=['x','yy','z'], offset=2., label=13)
+    o = aba.BaseAudioTime(time_res=0.001, data=d, filename='x', offset=2., label=13)
     return o, d
 
 @pytest.fixture
 def five_time_stamped_wave_files():
+    """ Each wav file is 0.5 second long. The sampling rate is 1 kHz."""
     N = 5
     path_to_tmp = os.path.join(path_to_assets,'tmp')
     folder = path_to_tmp + '/five_time_stamped_wave_files/'
@@ -473,7 +510,7 @@ def five_time_stamped_wave_files():
     for i in range(N):
         fname = 'empty_HMS_12_ 5_ {0}__DMY_23_ 2_84.wav'.format(i)
         full_path = os.path.join(folder, fname)
-        a = Waveform(rate=1000, data=np.zeros(500))
+        a = Waveform(rate=1000, data=np.random.uniform(size=500))
         a.to_wav(full_path)
         files.append(full_path)
 
@@ -487,21 +524,19 @@ def spectr_settings():
     j = '{"spectrogram": {"type":"MagSpectrogram", "rate": "20 kHz",\
         "window": "0.1 s", "step": "0.025 s",\
         "window_func": "hamming", "freq_min": "30Hz", "freq_max": "3000Hz",\
-        "duration": "1.0s", "resample_method": "scipy", "normalize_wav": "False"}}'
+        "duration": "1.0s", "resample_method": "scipy", "normalize_wav": "False", "decibel": "true",\
+        "transforms": [{"name":"enhance_signal", "enhancement":1.0}, {"name":"adjust_range", "range":"(0,1)"}],\
+        "waveform_transforms": [{"name":"add_gaussian_noise", "sigma":0.2}] }}'
     return j
 
-
 @pytest.fixture
 def sample_data():
-    data = np.vstack([np.zeros((10,512,512,1)), np.ones((10,512,512,1))])
+    data = np.vstack([np.zeros((10,64,64,1)), np.ones((10,64,64,1))])
     labels = np.concatenate([np.array([[1,0] for i in range(10)]), np.array([[0,1] for i in range(10)])])
-    
     return (data, labels)
 
 @pytest.fixture
 def sample_data_1d():
     data = np.vstack([np.zeros((10,20000,1)), np.ones((10,20000,1))])
     labels = np.concatenate([np.array([[1,0] for i in range(10)]), np.array([[0,1] for i in range(10)])])
-    
-
     return (data, labels)
\ No newline at end of file
