@@ -18,7 +18,7 @@ path_to_tmp = os.path.join(path_to_assets,'tmp')
 
 @pytest.fixture
 def batch():
-    data = np.vstack([np.zeros((10,512,512)), np.ones((3,512,512)),np.zeros((10,512,512)), np.ones((3,512,512)),np.zeros((4,512,512))])
+    data = np.vstack([np.zeros((10,8,8)), np.ones((3,8,8)),np.zeros((10,8,8)), np.ones((3,8,8)),np.zeros((4,8,8))])
     support = np.array([('file_1.wav', i*0.5) for i in range(30)],dtype=[('filename', '|S10'), ('offset', '>f4')])
     return data, support
 
@@ -27,6 +27,7 @@ def mock_audio_loader(batch):
     data, support = batch
     audio_loader = MagicMock()
     audio_loader.current = 0
+    audio_loader.batch_size = 1
 
     def mock_next(self):
         if self.current == 30: 
@@ -167,6 +168,19 @@ def test_process_batch_without_avg_and_group(batch):
     assert detections == expected_detections
 
 
+def test_process_batch_without_avg_and_group_multiple_thresholds(batch):
+    data, support = batch
+    transformed_data, transformed_support = transform_batch(data,support)
+    model = CNNInterface.load_model_file(os.path.join(path_to_assets, "test_model.kt"), os.path.join(path_to_tmp, "tmp_model"))
+
+    expected_detections = [('file_1.wav', 5.0, 3.0, 1.0), ('file_1.wav', 5.5, 3.0, 1.0), ('file_1.wav', 6.0, 3.0, 1.0), ('file_1.wav', 11.5, 3.0, 1.0), ('file_1.wav', 12.0, 3.0, 1.0), ('file_1.wav', 12.5, 3.0, 1.0)]
+    detections = process_batch(batch_data=transformed_data, batch_support_data=transformed_support, model=model, buffer=1.0, step=0.5, threshold=[0.5,0.75,1.1], win_len=1, group=False)
+
+    assert detections[0] == expected_detections
+    assert detections[1] == expected_detections
+    assert len(detections[2]) == 0
+
+
 def test_process_batch_with_avg_and_group(batch):
     data, support = batch
     transformed_data, transformed_support = transform_batch(data,support)
@@ -231,10 +245,10 @@ def test_transform_batch(batch):
     data, support = batch
     transformed_data, transformed_support = transform_batch(data,support)
 
-    expected_data = np.vstack([np.zeros((10,512,512)), np.ones((3,512,512)),np.zeros((10,512,512)), np.ones((3,512,512)),np.zeros((4,512,512))])
+    expected_data = np.vstack([np.zeros((10,8,8)), np.ones((3,8,8)),np.zeros((10,8,8)), np.ones((3,8,8)),np.zeros((4,8,8))])
     expected_support = np.array([('file_1.wav', str(i*0.5)) for i in range(30)],dtype='<U32')
 
-    assert transformed_data.shape == (30,512,512)
+    assert transformed_data.shape == (30,8,8)
     assert transformed_support.shape == (30,2)
     assert np.array_equal(transformed_data, expected_data)
     assert np.array_equal(transformed_support, expected_support)
